"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"

InspectorView subclass:#DictionaryInspectorView
	instanceVariableNames:'keys'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Inspector'
!

!DictionaryInspectorView class methodsFor:'documentation'!

copyright
"
 COPYRIGHT (c) 1993 by Claus Gittinger
	      All Rights Reserved

 This software is furnished under a license and may be used
 only in accordance with the terms of that license and with the
 inclusion of the above copyright notice.   This software may not
 be provided or otherwise made available to, or used by, any
 other person.  No title to or ownership of the software is
 hereby transferred.
"
!

documentation
"
    a modified Inspector for Dictionaries

    [author:]
        Claus Gittinger
"
! !

!DictionaryInspectorView methodsFor:'menu'!

fieldMenu
    |labels selectors|

    inspectedObject == Smalltalk ifTrue:[
        labels := #(
                       'inspect'
                       'inspect key'
                       'basicInspect'
                       'references'
                       '-'
                       'add key'
                       'remove key'
                   ).
        selectors := #(
                       doInspect 
                       doInspectKey 
                       doBasicInspect 
                       doReferences 
                       nil 
                       doAddKey 
                       doRemoveKey
                      ).
    ] ifFalse:[
        labels := #(
                       'inspect'
                       'inspect key'
                       'basicInspect'
                       '-'
                       'add key'
                       'remove key'
                   ).
        selectors := #(
                       doInspect 
                       doInspectKey 
                       doBasicInspect 
                       nil 
                       doAddKey 
                       doRemoveKey
                      ).
    ].

    monitorProcess isNil ifTrue:[
        labels := labels , #('-' 'start monitor').
        selectors := selectors , #(nil #doStartMonitor).
    ] ifFalse:[
        labels := labels , #('-' 'stop monitor').
        selectors := selectors , #(nil #doStopMonitor).
    ].

    ^ PopUpMenu
         labels:(resources array:labels)
         selectors:selectors

    "Modified: 12.4.1996 / 13:45:32 / cg"
! !

!DictionaryInspectorView methodsFor:'menu actions'!

doAddKey
    "add a key"

    |keyName key|

    keyName := Dialog request:'key to add:' initialAnswer:''.
    keyName notEmpty ifTrue:[
	key := keyName asSymbol.
	(inspectedObject includesKey:key) ifFalse:[
	    inspectedObject at:key put:nil.
	    selectedLine := nil.
	    inspectedObject changed.
	    self inspect:inspectedObject. "force list update"
	]
    ]
!

doInspectKey
    "inspect selected items key"

    selectedLine notNil ifTrue:[
	selectedLine ~~ 1 ifTrue:[
	    (keys at:selectedLine - 1) inspect
	].
    ]
!

doReferences
    "show users of selected key (i.e. global)"

    |k|

    selectedLine notNil ifTrue:[
	selectedLine == 1 ifFalse:[
	    k := (keys at:selectedLine - 1).
	    SystemBrowser browseReferendsOf:k asSymbol
	].
    ]
!

doRemoveKey
    "remove selected item from keys"

    |key|

    selectedLine == 1 ifFalse:[
	key := (keys at:selectedLine - 1).
	(inspectedObject includesKey:key) ifTrue:[
	    listView cursor:(Cursor wait).
	    inspectedObject removeKey:key.
	    keys := nil.
	    selectedLine := nil.
	    inspectedObject changed.
	    listView cursor:(Cursor hand).
	    self inspect:inspectedObject. "force list update"
	].
    ]
! !

!DictionaryInspectorView methodsFor:'private'!

fieldList 
    "return a list of names for the selectionlist. Leave hasMore as
     true, if a '...' entry should be added."

    |aList|

    inspectedObject isNil ifTrue:[^ #()].

    keys := inspectedObject keys asSortedCollection:[:a :b | a printString < b printString].
    aList := keys collect:[:k | k isSymbol ifTrue:[
                                                k printString
                                               ] ifFalse:[
                                                k displayString
                                               ]
                                         ].
    aList := aList asOrderedCollection.
    aList addFirst:'self'.
    ^ aList

    "Modified: 9.2.1996 / 21:16:07 / cg"
!

release 
    "release inspected object"

    keys := nil.
    super release
! !

!DictionaryInspectorView methodsFor:'user interaction'!

doAccept:theText
    "accept value for selected item"

    |value|

    value := Compiler evaluate:theText receiver:inspectedObject notifying:workspace.

    selectedLine notNil ifTrue:[
	selectedLine == 1 ifFalse:[
	    inspectedObject at:(keys at:selectedLine - 1) put:value.
	    inspectedObject changed
	].
    ]
!

valueAtLine:lineNr
    "helper - return the value of the selected entry"

    |key|

    lineNr == 1 ifTrue:[
        ^ inspectedObject
    ].
    key := (keys at:lineNr - 1).
    ^ inspectedObject at:key ifAbsent:nil.

    "Modified: 22.2.1996 / 17:56:42 / cg"
! !

!DictionaryInspectorView class methodsFor:'documentation'!

version
    ^ '$Header: /cvs/stx/stx/libtool/Attic/DictInspV.st,v 1.22 1996-04-25 16:32:16 cg Exp $'
! !
